/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_OBJECTLOCATION_H
#define DPC_IMPL_TYPE_OBJECTLOCATION_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint32.h"
#include "dpc/impl_type_slpoint.h"
#include "impl_type_float.h"
#include "impl_type_uint8.h"

namespace dpc {
struct ObjectLocation {
    ::UInt32 object_id{ 0U };
    ::dpc::SLPoint reference_point2corridor_frenet;
    ::Float relative_center_x{ 0.0F };
    ::Float relative_center_y{ 0.0F };
    ::Float size_width{ 0.0F };
    ::Float size_length{ 0.0F };
    ::UInt8 classification{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(object_id);
        fun(reference_point2corridor_frenet);
        fun(relative_center_x);
        fun(relative_center_y);
        fun(size_width);
        fun(size_length);
        fun(classification);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(object_id);
        fun(reference_point2corridor_frenet);
        fun(relative_center_x);
        fun(relative_center_y);
        fun(size_width);
        fun(size_length);
        fun(classification);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("object_id", object_id);
        fun("reference_point2corridor_frenet", reference_point2corridor_frenet);
        fun("relative_center_x", relative_center_x);
        fun("relative_center_y", relative_center_y);
        fun("size_width", size_width);
        fun("size_length", size_length);
        fun("classification", classification);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("object_id", object_id);
        fun("reference_point2corridor_frenet", reference_point2corridor_frenet);
        fun("relative_center_x", relative_center_x);
        fun("relative_center_y", relative_center_y);
        fun("size_width", size_width);
        fun("size_length", size_length);
        fun("classification", classification);
    }

    friend bool operator==(const ::dpc::ObjectLocation& lhs, const ::dpc::ObjectLocation& rhs) noexcept
    {
        return (lhs.object_id == rhs.object_id) && (lhs.reference_point2corridor_frenet == rhs.reference_point2corridor_frenet) && (fabs(static_cast<double>(lhs.relative_center_x - rhs.relative_center_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.relative_center_y - rhs.relative_center_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.size_width - rhs.size_width)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.size_length - rhs.size_length)) < DBL_EPSILON) && (lhs.classification == rhs.classification);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_OBJECTLOCATION_H
